/* The GIMP -- an image manipulation program
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * Globulise Plug-in for GIMP (by) ZERRO <zero@speed.bajt.com.pl>
 * based on: 
 * GIMP Plug-in Template
 * Copyright (C) 2000  Michael Natterer <mitch@gimp.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config.h"

#include <stdlib.h>  /*  For srand()  */
#include <time.h>    /*  For time()   */

#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>

#include "interface.h"
#include "main.h"

#include "plugin-intl.h"

/*  Constants  */

#define SCALE_WIDTH       180
#define SPIN_BUTTON_WIDTH  75
#define RANDOM_SEED_WIDTH 100


/*  Local function prototypes  */

static void   dialog_ok_callback     (GtkWidget *widget,
				      gpointer   data);
static void   dialog_pixmap_callback (GtkWidget *widget,
				      gpointer   data);

/*  Local variables  */

static PlugInUIVals *ui_state    = NULL;

static GtkWidget    *pixmap      = NULL;
static GtkWidget    *coordinates = NULL;

static gboolean      run         = FALSE;


/*  Public functions  */

gboolean
dialog (gint32              image_ID,
	GimpDrawable       *drawable,
	PlugInVals         *vals,
	PlugInImageVals    *image_vals,
	PlugInDrawableVals *drawable_vals,
	PlugInUIVals       *ui_vals)
{
  GtkWidget *dlg;
  GtkWidget *main_vbox;
  GtkWidget *frame;
  GtkWidget *table;
  GtkObject *adj;
  gint       row;

  ui_state = ui_vals;

  gimp_ui_init (PLUGIN_NAME, TRUE);

  dlg = gimp_dialog_new (_("Globulise"), PLUGIN_NAME,
			 gimp_standard_help_func, PLUGIN_NAME".html",
			 GTK_WIN_POS_MOUSE,
			 FALSE, TRUE, FALSE,

			 _("OK"), dialog_ok_callback,
			 NULL, NULL, NULL, TRUE, FALSE,
			 _("Cancel"), gtk_widget_destroy,
			 NULL, 1, NULL, FALSE, TRUE,

			 NULL);

  gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
		      GTK_SIGNAL_FUNC (gtk_main_quit),
		      NULL);

  /*  Initialize tooltips  */
  gimp_help_init ();

  main_vbox = gtk_vbox_new (FALSE, 4);
  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 6);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dlg)->vbox), main_vbox);
  

  /*  gimp_scale_entry_new() examples  */

  frame = gtk_frame_new (_("Settings"));
  gtk_box_pack_start (GTK_BOX (main_vbox), frame, FALSE, FALSE, 0);
  gtk_widget_show (frame);

  table = gtk_table_new (3, 3, FALSE);
  gtk_container_set_border_width (GTK_CONTAINER (table), 4);
  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
  gtk_container_add (GTK_CONTAINER (frame), table);
  gtk_widget_show (table);

  row = 0;

  adj = gimp_scale_entry_new (GTK_TABLE (table), 0, row++,
			      _("Fold [%]"), SCALE_WIDTH, SPIN_BUTTON_WIDTH,
			      vals->fold, 0, 50, 1, 5, 0,
			      TRUE, 0, 0,
			      _("Fold [%]"), NULL);
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
		      GTK_SIGNAL_FUNC (gimp_int_adjustment_update),
		      &vals->fold);

  adj = gimp_scale_entry_new (GTK_TABLE (table), 0, row++,
			      _("Number of stripes"), SCALE_WIDTH, SPIN_BUTTON_WIDTH,
			      vals->nr_of_stripes, 0, 30, 1, 3, 0,
			      TRUE, 0, 0,
			      _("Number of stripes"), NULL);
  gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
		      GTK_SIGNAL_FUNC (gimp_int_adjustment_update),
		      &vals->nr_of_stripes);


  /*  Show the main containers  */

  gtk_widget_show (main_vbox);
  gtk_widget_show (dlg);

  gtk_main ();

  /*  Free tooltips  */
  gimp_help_free ();

  return run;
}


/*  Private functions  */

static void
dialog_ok_callback (GtkWidget *widget,
		    gpointer   data)
{
  /*  Save ui values  */
  if (coordinates)
    ui_state->chain_active = GIMP_COORDINATES_CHAINBUTTON (coordinates)->active;

  run = TRUE;

  gtk_widget_destroy (GTK_WIDGET (data));
}

static void
dialog_pixmap_callback (GtkWidget *widget,
			gpointer   data)
{
  if (pixmap)
    gimp_pixmap_set (GIMP_PIXMAP (pixmap), data);
}
