/* The GIMP -- an image manipulation program
 * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
 *
 * 
 * Globulise Plug-in for GIMP (by) ZERRO <zero@speed.bajt.com.pl>
 * based on:
 * GIMP Plug-In Template
 * Copyright (C) 2000  Michael Natterer <mitch@gimp.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "config.h"

#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>
#include <unistd.h>
#include <stdio.h>
#include <math.h>


#include "interface.h"
#include "main.h"
#include "render.h"

#include "plugin-intl.h"


/*  Constants  */

#define PROCEDURE_NAME   "globulise"

#define DATA_KEY_VALS    "globulise"
#define DATA_KEY_UI_VALS "globulise_ui"

#define PARASITE_KEY     "globulise-options"


/*  Local function prototypes  */

static void   query (void);
static void   run   (gchar      *name,
		     gint        nparams,
		     GimpParam  *param,
		     gint       *nreturn_vals,
		     GimpParam **return_vals);


/*  Local variables  */

const PlugInVals default_vals =
{
  4,
  4,
};

const PlugInImageVals default_image_vals =
{
  0
};

const PlugInDrawableVals default_drawable_vals =
{
  0
};

const PlugInUIVals default_ui_vals =
{
  TRUE
};

static PlugInVals         vals;
static PlugInImageVals    image_vals;
static PlugInDrawableVals drawable_vals;
static PlugInUIVals       ui_vals;


GimpPlugInInfo PLUG_IN_INFO =
{
  NULL,  /* init_proc  */
  NULL,  /* quit_proc  */
  query, /* query_proc */
  run,   /* run_proc   */
};

MAIN ()

static void
query (void)
{
  static GimpParamDef args[] =
  {
    { GIMP_PDB_INT32,    "run_mode",   "Interactive, non-interactive"       },
    { GIMP_PDB_IMAGE,    "image",      "Input image"                        },
    { GIMP_PDB_DRAWABLE, "drawable",   "Input drawable"                     },
    { GIMP_PDB_INT32,    "dummy",      "dummy1"                             },
    { GIMP_PDB_INT32,    "dummy",      "dummy2"                             },
    { GIMP_PDB_INT32,    "dummy",      "dummy3"                             },
    { GIMP_PDB_INT32,    "time_seed",  "Current time is random seed (1, 0)" },
    { GIMP_PDB_INT32,    "seed_value", "Seed value (if time_seed == 0)"     }
  };
  static gint nargs = sizeof (args) / sizeof (args[0]);

  gimp_plugin_domain_register (PLUGIN_NAME, LOCALEDIR);
  gimp_plugin_help_register (DATADIR"/help");

  gimp_install_procedure (PROCEDURE_NAME,
			  "Blurb",
			  "Help",
			  "ZERRO <zero@speed.bajt.com.pl>",
			  "ZERRO <zero@speed.bajt.com.pl>",
			  "2000",
			  N_("<Image>/Filters/Misc/Globulise"),
			  "RGB*, GRAY*, INDEXED*",
			  GIMP_PLUGIN,
			  nargs, 0,
			  args, NULL);
}

static void
run (gchar      *name, 
     gint        n_params, 
     GimpParam  *param, 
     gint       *nreturn_vals,
     GimpParam **return_vals)
{
  static GimpParam   values[1];
  gint32             image_ID;
  GimpRunModeType    run_mode;
  GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
  GimpDrawable      *drawable;


  *nreturn_vals = 1;
  *return_vals  = values;


  run_mode = param[0].data.d_int32;
  image_ID = param[1].data.d_int32;
  drawable = gimp_drawable_get (param[2].data.d_drawable);

  /*  Initialize with default values  */
  vals          = default_vals;
  image_vals    = default_image_vals;
  drawable_vals = default_drawable_vals;
  ui_vals       = default_ui_vals;

  if (strcmp (name, PROCEDURE_NAME) == 0)
    {
      switch (run_mode)
	{
	case GIMP_RUN_NONINTERACTIVE:
	  if (n_params != 8)
	    {
	      status = GIMP_PDB_CALLING_ERROR;
	    }
	  else
	    {
	      vals.fold     = param[3].data.d_int32;
	      vals.nr_of_stripes     = param[4].data.d_int32;
	    }
	  break;

	case GIMP_RUN_INTERACTIVE:
	  /*  Possibly retrieve data  */
	  gimp_get_data (DATA_KEY_VALS,    &vals);
	  gimp_get_data (DATA_KEY_UI_VALS, &ui_vals);
	  INIT_I18N_UI ();
	  
	    if (GIMP_PDB_CANCEL == status)
	    	printf("1:Kansel\n");
	    if (status == GIMP_PDB_SUCCESS)
		printf("1:Sukces\n");
		
	  if (! dialog (image_ID, drawable,
			&vals, &image_vals, &drawable_vals, &ui_vals))
	    {
	      status = GIMP_PDB_CANCEL;
	    }
	    
	    if (GIMP_PDB_CANCEL == status)
	    	printf("2:Kansel\n");
	    if (status == GIMP_PDB_SUCCESS)
		printf("2:Sukces\n");
	  break;

	case GIMP_RUN_WITH_LAST_VALS:
	  /*  Possibly retrieve data  */
	  gimp_get_data (DATA_KEY_VALS, &vals);
	  break;

	default:
	  break;
	}
    }
  else
    {
      status = GIMP_PDB_CALLING_ERROR;
    }
    
    
      
  if (status == GIMP_PDB_SUCCESS)
    {
      globulise (drawable);
      render (image_ID, drawable, &vals, &image_vals, &drawable_vals);

      if (run_mode != GIMP_RUN_NONINTERACTIVE)
	gimp_displays_flush ();

      if (run_mode == GIMP_RUN_INTERACTIVE)
	{
	  gimp_set_data (DATA_KEY_VALS,    &vals,    sizeof (vals));
	  gimp_set_data (DATA_KEY_UI_VALS, &ui_vals, sizeof (ui_vals));
	}

      gimp_drawable_detach (drawable);
    }
    

  values[0].type = GIMP_PDB_STATUS;
  values[0].data.d_status = status;
}

void globulise (GimpDrawable      *drawable)
{
gint32	width, height;
gint	bytes, licznik;
gint	row, col;
GimpPixelRgn  srcPR, destPR;
guchar       *src_row, *dest_row, *dest_row2, *dest_row3;
int bufor[BUFFER_SIZE];
int i, j, nic, NrOfSections, SectionWidth, ShapeWidth, BeforeShape1Width, BeforeShape2Width, new_width, new_width_max;
double        coef = 0;
gint32        border_width;


/* moje start */

          width = drawable->width;
          height = drawable->height;
          bytes = drawable->bpp;
/* START making fold*/
          border_width = ((float)vals.fold/100)*width;
          gimp_pixel_rgn_init (&srcPR, drawable, 0, 0, width, height, FALSE, FALSE);
          gimp_pixel_rgn_init (&destPR, drawable, 0, 0, border_width, height, TRUE, TRUE);
src_row  = (guchar *) g_malloc (width * bytes);
dest_row = (guchar *) g_malloc (width * bytes);
dest_row2 = (guchar *) g_malloc (width * bytes);
dest_row3 = (guchar *) g_malloc (width * bytes);
      NrOfSections = vals.nr_of_stripes;
      gimp_progress_init (_("Globulise..."));
      for (row = 0; row < height; row++)
	{
	  gimp_pixel_rgn_get_row (&srcPR, src_row, 0, row, width); 
    	  for (col=0; col < border_width*bytes; col++)
    	  {
    	  	dest_row[col] = ((src_row[col]*((float)col/((float)border_width*(float)bytes)))+(src_row[col+(width-border_width)*bytes]*(1-((float)col/((float)border_width*(float)bytes)))));
    	  }
    	  for (col=border_width*bytes; col < width*bytes; col++)
    	  {
    	  	dest_row[col] = src_row[col];
    	  }
    	  if ((row % 15) == 0)
	    	  gimp_progress_update ((double) row / (double) height);
/* END making fold*/
	  new_width_max = (width-border_width);
	  new_width_max = new_width_max - (new_width_max % NrOfSections);
	  /*printf ("new_width_max=%d\t",new_width_max);	*/

/* START stretching */

	  i = row - (height/2);
	  nic = col;
	  if (i<0)
	  	i = -i;
	  coef = cos (((double)i/((double)height/2))*(3.14/2));
	  new_width = (width-border_width) * coef;
	  new_width = new_width - (new_width%NrOfSections);
	  coef = (float)(new_width)/(float)(width-border_width);
    	  for (col=0; col < new_width; col++)
    	  {
    	  	for (j=0; j<bytes; j++)
    	  	{
    	  		bufor[j] = 0;
    	  	}
    	  	licznik = 0;
    	  	for (j=((int)(col/coef)); j<((int)((col+1)/coef)); j++)
    	  	{
    	  		licznik++;
    	  		for (i=0; i<bytes; i++)
    	  		{
    	  			bufor[i] = bufor[i] + dest_row[(j*bytes)+i];
    	  		}
    	  	}
    	  	for (i=0; i<bytes; i++)
    	  	{
    	  		dest_row2[(col*bytes)+i] = (int)(bufor[i]/licznik);
    	  	}
    	  }
    	  for (col=new_width*bytes; col < (width-border_width)*bytes; col++)
    	  {
    	  	dest_row2[col] = 0;
    	  }
/* STOP stretching  */

/* START making sections */
	  SectionWidth = (int)((new_width_max)/NrOfSections);
	  ShapeWidth = (int)(new_width/NrOfSections);
	  licznik = 0;
	  BeforeShape1Width = (int)((SectionWidth - ShapeWidth)/2);
	  BeforeShape2Width = SectionWidth - (BeforeShape1Width + ShapeWidth);
	  for (i=0; i<NrOfSections; i++)
	  {
	  	for (col=i*SectionWidth; col<(i*SectionWidth)+BeforeShape1Width; col++)
	  	{
	  		for (j=0; j<bytes; j++)
	  		{
	  			dest_row3[j+(col*bytes)]=0;
	  		}
	  	}
	  	for (col=(i*SectionWidth)+BeforeShape1Width; col<(i*SectionWidth)+BeforeShape1Width+ShapeWidth; col++)
	  	{
	  	        for (j=0; j<bytes; j++)
	  		{
	  			dest_row3[j+(col*bytes)]=dest_row2[licznik++];
	  		}
	  	}
	  	for (col=(i*SectionWidth)+BeforeShape1Width+ShapeWidth; col<(i*SectionWidth)+BeforeShape1Width+ShapeWidth+BeforeShape2Width; col++)
	  	{
	  	        for (j=0; j<bytes; j++)
	  		{
	  			dest_row3[j+(col*bytes)]=0;
	  		}
	  	}
	  }
	  for (col = new_width_max; col<width-border_width; col++)
	  {
	  	        for (j=0; j<bytes; j++)
	  		{
	  			dest_row3[j+(col*bytes)]=0;
	  		}	  	
	  }
/* STOP making sections  */
    	  gimp_pixel_rgn_set_row (&destPR, dest_row3, 0, row, width);
	}
      g_free (src_row);
      g_free (dest_row);
      g_free (dest_row2);
      g_free (dest_row3);
      gimp_layer_resize (drawable->id, width-border_width, height, 0, 0);
      gimp_drawable_flush (drawable); 
      gimp_drawable_merge_shadow (drawable->id, TRUE); 
      gimp_drawable_update (drawable->id, 0, 0, width, height);    
/* moje KONIEC */	  
	
}

